# run.py 
import pylibm
import os
import time
import random

import esproto
from esproto.common_pb2 import DEV_SIGN_IN, DEV_OBJ_GET, DEV_OBJ_CHG, OK, Val, \
    DEV_SENS_VAL_SET, DEV_ACTR_VAL_INIT, DEV_ACTR_VAL_RECV, DEV_ACTR_VAL_ACK, \
    ERROR_MSG, ErrorMsg
from esproto.devapi.system_pb2 import SignInReq, SignInResp
from esproto.devapi.object_pb2 import ObjGetReq, ObjGetResp, Obj
from esproto.devapi.sensor_pb2 import SensValSetReq, ActrValInitReq, ActrValRecvMsg


os.system('modprobe w1-gpio')
os.system('modprobe w1-therm')

base_dir = '/sys/bus/w1/devices/'
device_id = '28-3ce104575e49'
device_file = base_dir + device_id + '/w1_slave'


def read_temp_raw():
    f = open(device_file, 'r')
    lines = f.readlines()
    f.close()
    return lines


def read_temp() -> float:
    lines = read_temp_raw()
    while lines[0].strip()[-3:] != 'YES':
        time.sleep(0.2)
        lines = read_temp_raw()
    equals_pos = lines[1].find('t=')
    if equals_pos != -1:
        temp_string = lines[1][equals_pos+2:]
        temp_c = float(temp_string) / 1000.0
        return temp_c


ctx = pylibm.mc_ctx()
conn = pylibm.mc_conn(ctx)


def __build_login_message(serial):
    print("Prepare singIn request")
    print(f"  serial: { serial }")

    r = SignInReq()
    r.serial_num = serial
    data = bytes(r.SerializeToString())

    m = pylibm.mc_msg(ctx, len(data))
    m.set_data(data)

    msg_type = DEV_SIGN_IN
    msg_id = random.randrange(1000000)

    return m, msg_type, msg_id


op_timeout = 15000
login_request_id: int = 0
request_objects_request_id: int =0
ready: bool = False

obj_by_id = dict()
obj_by_serial = dict()

def login():
    # login
    global login_request_id

    m, msg_type, login_request_id = __build_login_message("device-1") # store login request ID to identify it in callback
    conn.req_msg(m, msg_type, login_request_id, op_timeout)
    print(f"Sing In request Sent: { login_request_id }")


def __build_request_objects_message():
    r = ObjGetReq()
    r.flags.get_sens = True
    r.flags.get_actr = False
    r.flags.get_strm = False
    r.flags.get_obj_conf = True

    data = bytes(r.SerializeToString())
    m = pylibm.mc_msg(ctx, len(data))

    m.set_data(data)
    msg_type = DEV_OBJ_GET
    msg_id = random.randrange(1000000)

    return m, msg_type, msg_id


def request_objects():
    global request_objects_request_id

    m, msg_type, request_objects_request_id = __build_request_objects_message()
    conn.req_msg(m, msg_type, request_objects_request_id, op_timeout)
    print(f"Request objects request Sent: { request_objects_request_id }")


def __build_set_sens_message(obj, val: float):
    r = SensValSetReq()
    v = r.val.add()
    v.id = obj.id
    v.val.type = Val.FLOAT
    v.val.float_val = val

    data = bytes(r.SerializeToString())

    m = pylibm.mc_msg(ctx, len(data))
    m.set_data(data)

    msg_type = DEV_SENS_VAL_SET
    msg_id = random.randrange(1000000)

    return m, msg_type, msg_id


def set_sens_val(obj, val: float):
    m, msg_type, reqId = __build_set_sens_message(obj, val)
    conn.req_msg(m, msg_type, reqId, op_timeout)
    print(f"Set sens value request sent: { reqId }")


def get_state(i):
    if i == 0:
        return "Closed"
    if i == 100: 
        return "Opening"
    if i == 200: 
        return "Open"
    if i == 300: 
        return "Closing"


def print_error(err_code):
    s = "Unknown error"
    if err_code == pylibm.MC_CONN_ERROR_OTHER:
        s = "Other connection error"
    elif err_code == pylibm.MC_CONN_ERROR_TIMEOUT:
        s = "Connection has timed out"
    elif err_code == pylibm.MC_CONN_ERROR_NOT_FOUND:
        s = "Host address was not found"
    elif err_code == pylibm.MC_CONN_ERROR_REFUSED:
        s = "TCP connection was refused by the host"
    elif err_code == pylibm.MC_CONN_ERROR_REJECTED:
        s = "Connection was rejected by the server"
    elif err_code == pylibm.MC_CONN_ERROR_CLOSED:
        s = "Connection was unexpectedly closed by the server"
    elif err_code == pylibm.MC_CONN_ERROR_ABORTED:
        s = "Connection was aborted by the client"
    elif err_code == pylibm.MC_CONN_ERROR_PROTOCOL:
        s = "Protocol error"
    elif err_code == pylibm.MC_CONN_ERROR_NETWORK:
        s = "Network error"
    return s


def on_msg_sent(c, id):
    print("Message with id = {0} has been sent".format(id))


def on_conn_error(c, err_code):
    print("Connection error ocuured: {0}".format(print_error(err_code)))


def on_conn_state_change(conn, new_state, old_state):
    print(f"New state { get_state(new_state) }, old state { get_state(old_state) }")

    if new_state == 200:
        login()
    #    msg_size = 100
    #    msg = pylibm.mc_msg(ctx, msg_size)
    #    message_data = bytes("I'm message", 'utf-8')
    #    msg.set_data(message_data) 
    #    conn.post_msg(msg, 1, 54564)

def on_resp_recv(conn, msg, type, result, req_id):
    global login_request_id
    global obj_by_id
    global obj_by_serial
    global ready

    print(f"Response for Request with ID = {req_id} received type={type}.")

    reply = None
    if type == ERROR_MSG:
        reply = ErrorMsg()
    elif type == DEV_SIGN_IN:
        reply = SignInResp()
    elif type == DEV_OBJ_GET:
        reply = ObjGetResp()

    if reply is not None:
        reply.ParseFromString(msg.get_data())
        print(reply)
    else:
        print(f"Type {type} is not handled")
        return

    if type == DEV_SIGN_IN:
        print("Logged in.")
        request_objects()

    elif type == DEV_OBJ_GET:
        print("Got object list")
        obj_by_id = dict()
        obj_by_serial = dict()

        for obj in reply.obj:
            if obj.type != Obj.SENS:
                continue

            obj_by_id[obj.id] = obj
            obj_by_serial[obj.sens_obj.serial_num] = obj

        print(obj_by_id)
        print(obj_by_serial)

        if "sensor-1-1" in obj_by_serial:
            ready = True
    else:
        print("WHY NO HANDLE?????")


def on_post_recv(conn, msg, type):
    print("Post message received type = {0}, bytes = {1}".format(type, msg.get_size()))


conn.set_connect_timeout(3600)
conn.set_msg_sent_cb(on_msg_sent)
conn.set_conn_error_cb(on_conn_error)
conn.set_resp_recv_cb(on_resp_recv)
conn.set_conn_state_cb(on_conn_state_change)
conn.set_post_recv_cb(on_post_recv)
conn.open("127.0.0.1", 45001)


# time.sleep(10)
while True:
    val = read_temp()
    if ready:
        set_sens_val(obj_by_serial["sensor-1-1"], val)
        print(f"Sent { val } to ES!")
    else:
        print(f"Got { val } but haven't sent to ES")
    time.sleep(1)